# defringe.cl -- do sky illumination correction & remove fringing from images
#
# Calls routines iterstat and addcomment.
#
# CHANGES:
# 95/01/27: created 
# 95/03/22: many modifications begun
# 96/02/26: allow saving of sky+fringe image when stopping task 

procedure defringe(rootname,imagelist,logfile,defstring,biasimage)

string rootname   {prompt="Rootname for output images and image list"}
string imagelist  {prompt="List of images to process (do not use an `@')"}
string logfile    {prompt="Name of log file"}
string defstring  {prompt="String to be appended to output images"}
string biasimage  {prompt="Zero level calibration image for ccdproc"}
bool   dofringe   {yes,prompt="Do fringe removal step?"}
real   gain       {prompt="Gain of CCD (for ccdclip in imcombine)"}
real   rdnoise    {prompt="Read noise of CCD (for ccdclip in imcombine)"}
bool   forcescale {no,prompt="Force recalculation of image scales?"}
bool   goon=yes   {prompt="Continue with task using displayed sky+fringe image?",mode="q"}
bool   ssave=no   {prompt="Save sky+fringe image?",mode="q"}
struct *list
struct *list2
struct *list3
struct *datelist

begin

string ddefstring, ffringename, outlist, sskyfringename, sskyname, biasimg
string rrootname, ddark, llogfile, bbadmask, eextension, normfile, img
string flatlist, flat, flatbase, meanfile, currentflat, base, mmaskname
string iimagelist, FlatImageList, fflatstr, check, ttplname, templist, flatdiv
string datefile, thedate, ickinfo, name, tempstring, otherstring, tempfile
real mean, nnumber, flatnorm, valmed, fringmed, ggain, rrdnoise, scale, scale2
real indiv_exptime, avg_exptime, sum_exptime
int sstartno, eendno, i, j, numpix, x, y, z
bool zzeroout, ttpl, sstdpars, go, fflatsave, force, ggoon

rrootname = rootname
iimagelist = imagelist
llogfile = logfile
ddefstring = defstring
biasimg = biasimage
ggain = gain
rrdnoise = rdnoise
force = forcescale

outlist = "list."//rrootname//ddefstring
sskyfringename=rrootname//ddefstring//".skyfringe.imh"	# sky+fringe image
ffringename=rrootname//ddefstring//".fringe.imh"	# fringe image
sskyname=rrootname//ddefstring//".sky.imh"		# sky-flat image

#The "go" parameter won't let the task run if you haven't loaded ccdred.
go=yes
if(!defpac("ccdred")) {go=no}
if(!deftask("imcalc")) {go=no}
if(!deftask("iterstat")) {go=no}
if(!deftask("addcomment")) {go=no}
if(go) {

#Set up log file, appending if it already exists
if(llogfile == "")
  llogfile=rrootname//".log"
print(" ", >> llogfile)

# Do iterstat to find each image's "true" median for use in scaling
print("===> Finding image scales and weights...")
tempfile = iimagelist
list2 = tempfile
sum_exptime = 0.
scale = 1
scale2 = 1
i = 0
while (fscan(list2,img) !=EOF) {
	fileroot(img,validim+)
	flat=fileroot.root//ddefstring//".imh"
	print(flat, >> outlist)
	i = i + 1
	force = forcescale
	imgets (img, "exptime")
	sum_exptime=sum_exptime+real(imgets.value)
	if (!force) {
		imgets (img,"SKYFRING",>&"dev$null")
		mean=real(imgets.value)
		if (mean == 0) {
			force = yes
			}
			else {
			scale = scale + 1./real(imgets.value)
			}
		}
	if (force) {
		iterstat.nsigrej=5
		iterstat.maxiter=10
	iterstat(img,nsigrej=5.,maxiter=10,lower=INDEF,upper=INDEF,print-,ver-)
		valmed = 1./iterstat.median
		hedit (img,"SKYFRING",valmed,add+,ver-,show-,update+)
		scale = scale + (1./valmed)
		}	
	}
sum_exptime=sum_exptime/i

# Use this list of images to make the sky-fringe frame
print("===> Creating template sky + fringe frame...")
imcombine("@"//tempfile, sskyfringename, sigma="", plfile="", combine="median", reject="ccdclip", project-, offsets="none", blank=0., scale="!SKYFRING", zero="none", weight="!SKYFRING", statsec="", lthresh=-100000, hthresh=100000, nlow=1, nhigh=1, nkeep=1, mclip+, lsigma=3, hsigma=3, rdnoise=rrdnoise, gain=ggain, snoise="0.", sigscale=0.1, pclip=-0.5, grow=0., logfile='STDOUT', >> llogfile)
hedit(sskyfringename,"exptime",sum_exptime,add+,ver-,show-,update+)
print("===> Displaying sky+fringe image for inspection.  If object residuals")
print("===> are present, you may wish to stop, create object masks, and rerun")
print("===> the task.  You can use makeobjmask on the non-defringed images to")
print("===> create object masks for them directly, or you can use shiftnadd on")
print("===> the non-defringed images, run mkobjmask on the skysubbed output,")
print("===> insert the object mask names into the non-defringed image headers,")
print("===> and then rerun this task so that the objects are masked out during")
print("===> the creation of the sky+fringe image.")
displ(sskyfringename,1)
ggoon=goon
if(!ggoon){
	print("Task aborted-- illumination correction & defringing not done.")
	if(!ssave)imdel(sskyfringename,ver-)
	del(outlist,ver-)
	return
	}

# "scale" is set equal to the average median of the input images.
# It is multiplied into the sky-fringe frame to keep fringe counts accurate.
scale = scale / i
imar (sskyfringename, "*", scale, sskyfringename)

# Insert title into fringe image header
hedit (sskyfringename, "title", "Sky+fringe image for "//tempfile, add+,ver-,show-,update+)

# Create sky illumination correction image
mkskycor(sskyfringename,sskyname,xboxmin=5.,xboxmax=0.25,yboxmin=5.,yboxmax=0.25)
#mkskycor(sskyfringename,out=sskyname,xboxmin=5.,xboxmax=0.25,yboxmin=5.,
#yboxmax=0.25)
hedit (sskyname, "title", "Sky illumination correction image for "//iimagelist, add+,ver-,show-,update+)

# Do illumination correction *on output images only*
imcopy("@"//iimagelist, "@"//outlist, ver-)
ccdproc("@"//outlist, illumcor+, illum=sskyname, zero=biasimg)
imcopy(sskyfringename, ffringename, ver-)
imar (ffringename, "-", sskyname, ffringename)

# Subtract accurately scaled fringe image from each input image, if desired
if(dofringe){
print("===> Subtracting scaled fringe image from each input image...")
imgets(ffringename,"exptime")
avg_exptime=real(imgets.value)
#list = iimagelist
list = outlist
while (fscan (list, name) !=EOF) {
#	x=strlen(name)
#	check=substr(name,x-3,x)
#	base=substr(name,1,x)
#	if(check==".imh")base=substr(name,1,x-4)
#	flat=base//ddefstring//".imh"
#	print(flat, >> outlist)
	imgets(name,"exptime")
	indiv_exptime=real(imgets.value)
	imgets(name,"SKYFRING")
	iterstat(name,nsigrej=5.,maxiter=10,lower=INDEF,upper=INDEF,print-,ver-)
# new scaling = new median from iterstat * old scaling * exptime / avg_exptime
	fringmed=iterstat.median*(real(imgets.value))*indiv_exptime/avg_exptime
	imren (name, "_"//name, ver-)
	imcalc ("_"//name//","//ffringename, name, "im1-im2*"//fringmed, ver-)
	imdel ("_"//name, ver-)
	hedit(name,"FRINGMED",fringmed,add+,ver-,show-,update+)
	addcomment(name,"Defringed using scaled copy of "//ffringename,ver-)
      print(name//" was defringed using "//ffringename//" scaled by "//fringmed)
	}
}

print("===> Wrapping up... ")
datefile = mktemp("date")
date(>>datefile)
datelist = datefile
print("finished at "//datelist, >> llogfile)
datelist = ""
del(datefile, ver-)
}
else {
	beep
	print("WARNING: one or more of the packages `noao.imred.ccdred' or")
	print("`stsdas.toolbox.imgtools' or `PHIIRS' are not loaded!")
	print("Please load them and restart this task.")
	}

beep

end

