# mkobjmask.cl -- script to interactively create object mask
#
# originally created 11 July 1992 by John S. Ward
# 930228: Now excludes zero exposure regions from computation of pixel sigma.
# 930726: Modified by MD to call sigmanorm and use makeobjmask.cl.
# 950208: Renamed mkobjmask. Various minor adjustments.
# 950209: inv parameter: prefer to have masks with objects=0 not 1.
# 960301: display object-masked image in interactive mode
# 960405: removed "inv" param and added interactive iteration mode
#
# Calls scripts makeobjmask, sigmanorm, iterstat, sctv

procedure mkobjmask(image,expname)

string	image		{prompt="Name of image for making mask"}
string	expname		{prompt="Exposure map (default=image//em)"}
string  suffix="objmask" {prompt="Suffix of output object mask"}

string	prefix="norm"	{prompt="Prefix for temporary normalized image"}
int     nsmooth {3,min=0,prompt="Filter size for boxcar smoothing (ODD values only)"}
int  subsample {1,min=1,prompt="Block averaging factor before median filtering"}
int	filtsize {15,min=1,prompt="Median filter size for local sky evaluation"}
int	ngrow	{1,min=0,prompt="Width of rings to grow around masked objects"}
bool	interact	{yes,prompt="Interactively examine normalized image?"}
real	threshold	{prompt="Cutoff point for replacement",mode="q"}
bool	redo    	{no,prompt="Redo with different parameters?",mode="q"}
int     z    {0,prompt="Number of image to save (zero to delete all)?",mode="q"}

int     i_nsmooth   {3,min=0,prompt="Filter size for boxcar smoothing (ODD values only)",mode="q"}
int     i_subsample {1,min=1,prompt="Block averaging factor before median filtering",mode="q"}
int     i_filtsize  {15,min=1,prompt="Median filter size for local sky evaluation",mode="q"}
int	i_ngrow	    {1,min=0,prompt="Width of rings to grow around masked objects",mode="q"}

begin

	string	img		# equals image
	string	ext		# equals image extension
	string	eexpname	# equals expname
	string	ssuffix	# equals suffix
	string	pprefix	# equals prefix
	real	th		# equals threshold
	real	recthresh	# recommended thresholding value
	real	minval		# minimum value in mask image
	real	maxexp		# peak exposure time
	real	flagval		# flag value
	bool	rredo		# iterate mask determination?
	int x    # number to append to tmp. images; increased by 1 each time
	int y    # dummy number; looped through to delete leftover images at end
	int zz   # dummy variable for `z'
	bool ook # dummy variable set equal to `ok' to keep loop going or not
	int nnsmooth, ssubsample, ffiltsize, nngrow	# dummy variables

# Get query parameters

	fileroot (image,validim+)
		img = fileroot.root
		ext = fileroot.extension
		if(ext!="")ext="."//ext
	eexpname = expname
		if(eexpname=="")eexpname=img//"em"
	ssuffix = suffix

	pprefix = prefix
	nnsmooth=nsmooth
	ssubsample=subsample
	ffiltsize=filtsize
	nngrow=ngrow

# Display initial image
	if (interact) {
		print ("Displaying image "//img//"...")
		display(img,1)
		}

# Normalize image to uniform pixel-to-pixel rms using sq.root of exposure map.
	print ("Normalizing image "//img//" to uniform RMS...")
	if(!access(pprefix//img//ext))sigmanorm(img,normimage=eexpname,prefix=pprefix)

# Set flag value at 1 DN below minimum pixel value in normalized image.

	minmax(pprefix//img,update-,ver-)
	minval = minmax.minval
	flagval = minval - 1.

# Identify regions with zero exposure time and exclude from sigma calculation by
# flagging them to the flagval and excluding them from the iterstat calculation.

	if(access("_zeroflag.imh"))imdel("_zeroflag.imh",ver-)
	if(access("_zeroflag.hhh"))imdel("_zeroflag.hhh",ver-)
	imcopy(eexpname,"_zeroflag",ver-)
	imreplace("_zeroflag",-1.,lower=INDEF,upper=0.)
	imreplace("_zeroflag",0.,lower=0.,upper=INDEF)
	imreplace("_zeroflag",flagval,lower=-1.,upper=-1.)
	imarith(pprefix//img,"+","_zeroflag","_norm"//img)
	imdelete ("_zeroflag",ver-)

# Calculate rms of masked, normalized image with iterative sigma rejection.
# Recommended threshold (to be applied to boxcar smoothed data) is approx.
# (4.5/nsmooth) x unsmoothed rms.

	iterstat("_norm"//img,nsigrej=5.,maxiter=10.,lower=minval,upper=INDEF,ver-,print-)
	imdelete ("_norm"//img,ver-)
	recthresh = 4.5 / nsmooth * iterstat.sigma
	print ("Recommended thresholding level for mask is ",recthresh)

# Allow user to interactively examine (unsmoothed) normalized image to set 
# threshold for masking.

if (interact) imexamine(pprefix//img,display="display(image='$1',frame=2,zs+)")

# If interactive, query for final threshold to adopt.
# If not interactive, use the default recommended threshold.

	if (interact) {
		beep
		threshold = recthresh
		th = threshold
	   } else {
		th = recthresh
	}

# Apply thresholding to image using makeobjmask.

	makeobjmask (pprefix//img,suffix=ssuffix,headlist="",subsample=subsample,filtsize=filtsize,nsmooth=nsmooth,threshtype="constant",constthresh=th,ngrow=ngrow,statsec="",checklimits+,inv+,ver-)
	imrename (pprefix//img//ssuffix//".pl",img//ssuffix//".pl",ver-)

# Display original image and mask if interact=yes.  
	if (interact) {
		sctv(img//ssuffix//".pl",3)
		imarith(img,"*",img//ssuffix//".pl","del_"//img)
		display("del_"//img,4)
		imdel("del_"//img,ver-)

# Allow iteration if desired.
		x=2
		beep
		rredo=redo
		while(rredo){
	  		if(x==2){
				imren (img//ssuffix//".pl",pprefix//img//ssuffix//"_1.pl",ver-)
				if (!access(img//ssuffix//"orig.pl")) imcopy (pprefix//img//ssuffix//"_1.pl",img//ssuffix//"orig.pl",ver-, >& "dev$null")
			}
		  print("Recommended thresholding level for mask is ",recthresh)
			th = threshold
			nnsmooth=i_nsmooth
			ssubsample=i_subsample
			ffiltsize=i_filtsize
			nngrow=i_ngrow
			makeobjmask (pprefix//img,suffix=ssuffix//"_"//str(x),headlist="",subsample=ssubsample,filtsize=ffiltsize,nsmooth=nnsmooth,threshtype="constant",constthresh=th,ngrow=nngrow,statsec="",checklimits+,inv+,ver-)
			sctv(pprefix//img//ssuffix//"_"//str(x)//".pl",3)
	     imar(img,"*",pprefix//img//ssuffix//"_"//str(x)//".pl","del_"//img)
			display("del_"//img,4)
			imdel("del_"//img,ver-)
# Another iteration?
			beep
			rredo=redo
			x+=1
		}
	}

# CLEAN UP
zz=1
if(interact)zz=z
if(zz!=0){
	imren(pprefix//img//ssuffix//"_"//str(zz)//".pl",img//ssuffix//".pl",ver-, >& "dev$null")
	if(access(img//ssuffix//"orig.pl")){
		print("Object masks "//img//ssuffix//".pl and "//img//ssuffix//"orig.pl created.")
	} else {
		print("Object mask "//img//ssuffix//".pl created.")
	}
}
else{
     print("No images saved.")
}
y=0
zz=0
if(interact){
    while(y<x){
        imdel(pprefix//img//ssuffix//"_"//str(y)//".pl",ver-, >& "dev$null")
        y+=1
    }
}
print("Next task to run is `masklet'.")
imdelete (pprefix//img,ver-)

beep

end

