# photcomp -- compare photometry between two images on the same coord system
#
# Task calls following other tasks: filecalc, fileroot.
#
# Output:
# 
# refimage//.pcomp//suffix	Individual-star & avg. photometry for all images
# refimage//.plog//suffix	Imexam log for refimage
# 
# 960402: created from irshift2.cl

procedure photcomp(refimage,compimage,suffix)

string refimage	      {prompt="Reference image"}
string compimage      {prompt="Comparison image"}
string suffix="1"     {prompt="Suffix string for output files"}

string reflog=""      {prompt="Ref. image imexam log (default=create new one)"}
bool center=yes       {prompt="Imexamine: center object in aperture?"}
bool background=yes   {prompt="Imexamine: fit and subtract background?"}
real radius=5.	      {prompt="Imexamine aperture photometry radius"}
real buffer=10.	      {prompt="Imexamine sky buffer"}
real width=10.	      {prompt="Imexamine sky annulus width"}
bool another          {prompt="Do another comparison image?",mode="q"}
struct *list

begin

string rrefimage, fflatlist, rrefcoords, bbadmask, rroundname
string sshiftlist, coarsefile, ooffname, ssuffix, iintname, ffracname
string pprefix, logoffname, sl, sx, img, iinterp, bboundary
string ccompimage, stringline1, stringline2, stringline1old, stringline2old
string stringline1tmp, stringline2tmp, georefin, geomapin
real p, pixscale, stcoordx, stcoordy, nrows, ncols, delx, dely, adelx, adely
real xold, yold, xxshiftold, yyshiftold, xshiftnew, yshiftnew, astcoordx
real astcoordy, nl, nx, x, y
int xref, yref, number, xtmp, ytmp, z
bool go, aanother

#The "go" parameter won't let the task run if you haven't loaded the ctio pkg.
go=yes
if(!deftask("filecalc") || (!deftask("fileroot"))) {go=no}
if(go) {

fileroot (refimage,validim+)
rrefimage = fileroot.root
fileroot (compimage,validim+)
ccompimage = fileroot.root
ssuffix = suffix

rimexam.center=center
rimexam.background=background
rimexam.radius=radius
rimexam.buffer=buffer
rimexam.width=width

del("_??g."//rrefimage,ver-,>&"dev$null")
del("_??g."//ccompimage,ver-,>&"dev$null")
del("_*refcomp",ver-,>&"dev$null")

display(ccompimage,2)
display(rrefimage,1)
	if(reflog==""){
		print(" ")
print("===) Choose an object that appears in both images, find its coordinates")
print("===) (type 'a') in frame 1 (the reference image), and repeat with as many")
print("===) objects as desired.  The imexam output log will be displayed for")
print("===> editing when done, in case mistakes were made.")
		print(" ")
		imexam(logfile="_log."//rrefimage, keep+)
		print(" ")
 print("===> In case mistakes were made, the imexam logfile can now be edited.")
		vi("_log."//rrefimage)
	} else {
		copy(reflog,"_log."//rrefimage,ver-)
	}

print("===> Finding comparison image object fluxes...")
	del("_pxy",ver-,>&"dev$null")
	filecalc ("_log."//rrefimage, "$1;$2;2", lines="1-9999", format="%8.2f %8.2f  %1d", calctype="real", >> "_pxy")
	imexam (ccompimage, defkey="a", imagecur="_pxy", keeplog+, logfile="_log."//ccompimage)

print("===> Finding percentage differences (reference-comparison)/reference...")
	filecalc ("_log."//rrefimage, "$4", lines="1-9999", calctype="real", >> "_mag."//rrefimage)
	filecalc ("_log."//ccompimage, "$4", lines="1-9999", calctype="real", >> "_mag."//ccompimage)
	join("_mag."//rrefimage//",_mag."//ccompimage//",_pxy", out="_mag.refcomp")
	print("#", >> "_temp.refcomp")
	print("# Col. 1. X position", >> "_temp.refcomp")
	print("# Col. 2. Y position", >> "_temp.refcomp")
	print("# Col. 3. "//rrefimage//" flux", >> "_temp.refcomp")
	print("# Col. 4. "//ccompimage//" flux", >> "_temp.refcomp")
	print("# Col. 5. % difference ("//rrefimage//"-"//ccompimage//")/"//rrefimage, >> "_temp.refcomp")
	filecalc ("_mag.refcomp", "$3;$4;$1;$2;($1-$2)/$1", lines="1-9999", format="%8.2f %8.2f %10.4f %10.4f %10.6f", calctype="real", >> "_temp.refcomp")
        print("# Output of `average' task: avg rms #_of_pts",>> "_temp.refcomp")
    fields("_temp.refcomp","5") | average (opt="new_sample", >> "_temp.refcomp")
 fields("_temp.refcomp","5") | graph (point+,mark="cross",szm=0.005,logx-,logy-)
	vi ("_temp.refcomp")
	fields("_temp.refcomp", "5") | average (opt="new_sample") | scan (x,y,z)
        print("# Output of `average' task: avg rms #_of_pts",>> "_temp.refcomp")
    fields("_temp.refcomp","5") | average (opt="new_sample", >> "_temp.refcomp")
        print("Avg. (ref-comp)/ref = "//x//" +- "//y)

# Clean up
	rename("_temp.refcomp", rrefimage//".pcomp"//ssuffix)
	rename("_log."//rrefimage, rrefimage//".plog"//ssuffix) 
	del("_??g."//ccompimage,ver-,>&"dev$null")
	del("_*refcomp",ver-,>&"dev$null")


#ANOTHER IMAGE?
aanother=another
while(aanother){
	fileroot(compimage)
	ccompimage=fileroot.root
	del("_??g."//ccompimage,ver-,>&"dev$null")
	display(ccompimage,2)
print("===> Finding comparison image object fluxes...")
	imexam (ccompimage, defkey="a", imagecur=coords, keeplog+, logfile="_log."//ccompimage)

print("===> Finding percentage differences (reference-comparison)/reference...")
	filecalc ("_log."//ccompimage, "$4", lines="1-9999", calctype="real", >> "_mag."//ccompimage)
	join("_mag."//rrefimage//",_mag."//ccompimage//",_pxy", out="_mag.refcomp")
	print("#", >> "_temp.refcomp")
	print("# Col. 1. X position", >> "_temp.refcomp")
	print("# Col. 2. Y position", >> "_temp.refcomp")
	print("# Col. 3. "//rrefimage//" flux", >> "_temp.refcomp")
	print("# Col. 4. "//ccompimage//" flux", >> "_temp.refcomp")
	print("# Col. 5. % difference ("//rrefimage//"-"//ccompimage//")/"//rrefimage, >> "_temp.refcomp")
	filecalc ("_mag.refcomp", "$3;$4;$1;$2;($1-$2)/$1", lines="1-9999", format="%8.2f %8.2f %10.4f %10.4f %10.6f", calctype="real", >> "_temp.refcomp")
       print("# Output of `average' task: avg rms #_of_pts", >> "_temp.refcomp")
    fields("_temp.refcomp","5") | average (opt="new_sample", >> "_temp.refcomp")
 fields("_temp.refcomp","5") | graph (point+,mark="cross",szm=0.005,logx-,logy-)
	vi ("_temp.refcomp")
	fields("_temp.refcomp", "5") | average (opt="new_sample") | scan (x,y,z)
        print("# Output of `average' task: avg rms #_of_pts",>> "_temp.refcomp")
    fields("_temp.refcomp","5") | average (opt="new_sample", >> "_temp.refcomp")
        print("Avg. (ref-comp)/ref = "//x//" +- "//y)

# Clean up
	del("_mag."//ccompimage,ver-,>&"dev$null")
	del("_??g."//ccompimage,ver-,>&"dev$null")
	type("_temp.refcomp", >> rrefimage//".pcomp"//ssuffix)
	del("_*refcomp",ver-,>&"dev$null")
# Next comparison image, if desired
	aanother=another
}

# Clean up
del("_mag."//rrefimage,ver-,>&"dev$null")
del("_pxy",ver-,>&"dev$null")
print("Files "//rrefimage//".pcomp"//ssuffix//" and "//rrefimage//".plog"//ssuffix//" have been created.")

}
else {
    print("WARNING: tasks `ctio.filecalc' and/or `phiirs.fileroot' not loaded!")
        print("Please load them and restart this task.")
        }

beep

end

