#shiftnadd.cl--script to find offsets for a set of CCD images and coadd them.
#
# Procedure:
# 1. Choose one image (which all images overlap) to be the reference image.
# 2. imexam the reference image & get the coordinates (using 'r' or 'a') of
#    ALL REFERENCE OBJECTS that are to be used.  
# 3. imexam all the images and get the coordinates (using 'r' or 'a' and 'n' 
#    to go to the next image) of ONLY THE BRIGHTEST or "BEST" reference object
#    in each.
# 4. You are asked for the X and Y coordinates of this "BEST" reference object.
#    the coords file is printed to jog your memory.
# 5. lintran is used to calculate the coarse shifts from just the "BEST" object
# 6. The coarse shifts are used as input to imcentroid, which finds the final
#    offsets using ALL reference objects available.  The imcentroid parameters
#    (e.g. boxsize) can be adjusted if deemed necessary.
# 
# Output:
# prefix.refcoords.outstring	registration objects' reference image coords
# prefix.allcoords.outstring	registration objects' coordinates in all images
# prefix.coarse.outstring	coarse shifts
# prefix.shiftnadd.outstring	final shifts, imcombine log, & misc. other info
# prefix.shiftfrac.outstring	fractional part of final shifts
# prefix.shiftint.outstring	integer part of final shifts
# prefix.roundoff.outstring	final shifts rounded off to integer shifts
# coaddname.imh            	coadded image
# 
# CHANGES:
# 94/12/28: created from irshift.cl
# 95/01/27: added check for ctio package
# 95/03/21: started to add in exposure map stuff, many other changes
# 96/07/12: added output "shiftexact" file
# 96/07/22: added default airmass scaling option
# 96/07/23: fixed edge problem in sky-subracted image

procedure shiftnadd(flatlist,refimage,prefix,outstring,fracshift,coaddname,expmap)

string flatlist		{prompt="List of images for which to find offsets"}
string refimage		{prompt="Name of (flattened) reference image"}
string prefix		{prompt="Prefix string for output files"}
string outstring	{prompt="Suffix string for output files"}
bool fracshift	{prompt="Create fractional-pixel-offset images? (yes|no)"}
string coaddname	{prompt="Name for output coadded image"}
string expmap           {prompt="Name of output exposure map image"}
bool airscale		{prompt="Use default airmass scaling?"}

string shiftlist	{prompt="List of output, offset images",mode="q"}
real   extcoeff  {prompt="Extinction coefficient for default scalings",mode="q"}
real   airzero   {INDEF,min=0.,max=99.,prompt="Airmass to scale to? (INDEF=min. observed)",mode="q"}
bool combdata=no	{prompt="Combine shifts with earlier dataset? (yes|no)"}
string expname="exptime" {prompt="Header keyword for exposure time"}
int xref	{prompt="Reference object x coordinate (nearest integer)"}
int yref	{prompt="Reference object y coordinate (nearest integer)"}
real xshiftold	{prompt="Reference image x shift in earlier dataset"}
real yshiftold	{prompt="Reference image y shift in earlier dataset"}
real radius=5.	{prompt="Aperture photometry radius for imexamine"}
real buffer=10.	{prompt="Aperture photometry sky buffer for imexamine"}
real width=10.	{prompt="Aperture photometry sky annulus width for imexamine"}
real bs=9.	{prompt="Centering box size for final shift calculation"}
bool doshift=yes	{prompt="Do offset measuring step?"}
bool stdpars=yes	{prompt="Use std imcombine parameters for coadd?"}
bool makeavg=no		{prompt="Make pure coadd (comb=average,rej=none)?"}
pset addpars    	{prompt="Imcombine parameters"}
struct *list

begin

string subimg, explist, sect, check, img, eexpname, eexpmap, listimshift
string subimgold, rrefimage, fflatlist, rrefcoords, aallcoords, rroundname
string sshiftlist, coarsefile, ooffname, ooutstring, iintname, ffracname
string pprefix, logoffname, ccoaddname, sl, sx, coaddlist, offsetlist
string t_sigma, t_plfile, t_combine, t_reject, t_offsets, t_masktyp
string t_scale, t_weight, t_zero, t_statsec, t_expname, file_of_air
real t_maskval, t_blank, t_lthresh, t_hthresh, t_nlow, t_nhigh, t_nkeep
real t_lsigma, t_hsigma, t_rdnoise, t_gain, t_pclip, t_grow, t_sigscale
real weight, scalemed, stcoordx, stcoordy, nrows, ncols
real xold, yold, bbs, xxshiftold, yyshiftold, xshiftnew, yshiftnew, astcoordx
real astcoordy, avg, x, sig, nl, nx, y, eextcoeff, aairzero, air, texp
real effairmass, wtavg, nncombine, exp, exposure, sf, zf, testold, eexptime
int xmin, ymin, newx, newy, oldx, oldy, test1, test1old, test2, test2old, p
int sstartno, eendno, i, numpix, num, xdim, ydim, newxsize, newysize, ncoadds
bool ffracshift, ccombdata, go, dosh, aairscale

fflatlist = flatlist
rrefimage = refimage
pprefix = prefix
ooutstring = outstring
ffracshift = fracshift
if(ffracshift){
	sshiftlist = shiftlist
	}
ccoaddname = coaddname
eexpmap = expmap
if(eexpmap==""){
	fileroot(ccoaddname,validim+)
	eexpmap=fileroot.root//"em"
	}
aairscale=airscale
eextcoeff=0.
aairzero=100.
if(aairscale){
	eextcoeff=extcoeff
	aairzero=airzero
	if(aairzero==INDEF){
		print("...one moment please...")
                file_of_air=mktemp("foa")
                hsel("@"//fflatlist,"AIRMASS",yes, >> file_of_air)
                list=file_of_air
                aairzero=100.
                while (fscan (list,air) != EOF) {
                        aairzero=min(air,aairzero)
                }
                del(file_of_air,ver-,>&"dev$null")
                list=""
		}
	}

ccombdata = combdata
eexpname = expname
bbs = bs
#print("test1")

#Do not let task run if needed tasks are not loaded
go=yes
if(!deftask("filecalc")){go=no}
if(!deftask("fileroot")){go=no}
if(!deftask("imcalc")){go=no}
if(!deftask("iterstat")){go=no}
if(go){

ooffname = pprefix//".shiftexact."//ooutstring
logoffname = pprefix//".shiftnadd."//ooutstring
rrefcoords = pprefix//".refcoords."//ooutstring
aallcoords = pprefix//".allcoords."//ooutstring
coarsefile = pprefix//".coarse."//ooutstring
iintname = pprefix//".shiftint."//ooutstring
ffracname = pprefix//".shiftfrac."//ooutstring
rroundname = pprefix//".roundoff."//ooutstring

t_sigma = j_sigma       # imcombine parameters
t_plfile = j_plfile
t_combine = j_combine
t_reject = j_reject
t_masktyp = j_masktyp
t_maskval = j_maskval
t_blank = j_blank
t_scale = j_scale
t_weight = j_weight
t_zero = j_zero
t_statsec = j_statsec
t_expname = j_expname
t_lthresh = j_lthresh
t_hthresh = j_hthresh
t_nlow = j_nlow
t_nhigh = j_nhigh
t_nkeep = j_nkeep
t_lsigma = j_lsigma
t_hsigma = j_hsigma
t_rdnoise = j_rdnoise
t_gain = j_gain
t_sigscale = j_sigscale
t_pclip = j_pclip
t_grow = j_grow

if(doshift){
if(ccombdata){
print(" ")
print("===) So you wanna add some new images to an old coadd, eh?  Well, first make")
print("===) sure you're using the same reference image as that old set.  Then, take")
print("===) a look at the *.shiftnadd.* file from that set, find the fractional")
print("===) offsets made on the reference image, and enter them here:")
xxshiftold=xshiftold
yyshiftold=yshiftold
}

print(" ")
print("===) Get the coordinates, in the reference image, of all objects you wish to")
print("     use for registration, starting with the `best' reference object-- one")
print("     which appears in all images.")
print(" ")
	rimexam.radius=radius
	rimexam.buffer=buffer
	rimexam.width=width
	imexam(rrefimage, frame=1, logfile=rrefcoords, keep+)

#Interactively ID objects
print(" ")
print("===) Choose one object that appears in all images, and find the position")
print("===) (type 'a') of this best registration object in each frame (hit 'n' to go on")
print("===> to the next frame, and 'q' when done).  The file containing these positions")
print("===> will be displayed for editing when done, in case mistakes were made.")
print(" ")
	imexam("@"//fflatlist, frame=1, logfile=aallcoords, keep+)

print(" ")
print("===> In case mistakes were made, the coordinate file can now be edited.")
	vi(aallcoords)

# Find offsets
	fields (rrefcoords, "1-2") | scan (xref, yref)
	lintran(aallcoords, x1=0.001, y1=0.001, x2=xref, y2=yref, angle=180, > coarsefile)
print("===) Finding the final offsets, and saving to `shiftnadd' file.")
	del(logoffname, ver-, >& "dev$null")
	imcentroid("@"//fflatlist, rrefcoords, reference=rrefimage, shift=coarsefile, boxsize=bbs, > logoffname)
	del("temp.joinfile", ver-, >& "dev$null")
	del("temp.joinfile?", ver-, >& "dev$null")
# read in length of temp.joinfile0 and cut out last two lines
        copy(logoffname,"temp.joinfile0")
        count("temp.joinfile0") | scan (nl)
        nx=nl-2
        head("temp.joinfile0", nlines=nx, >> "temp.joinfile1")
# find line on which the string "X-s" appears and cut out that line & all before
        !grep -n X-s temp.joinfile1 > temp.joinfile2
        type("temp.joinfile2") | scan (sl)
        x=stridx(":",sl)
        sx=substr(sl,1,x)
        x=nx-real(sx)
        tail("temp.joinfile1", nlines=x, >> "temp.joinfile")
# Cut down on number of output files
        print(" ", >> logoffname)
        print("#Coarse offsets:", >> logoffname)
        print(" ", >> logoffname)
        type(coarsefile, >> logoffname)
        del(coarsefile, ver-)

if(ccombdata){
# Extract the new reference image shifts from the offsets file
  del("temp.joinfile?",ver-, >& "dev$null")
  grep(rrefimage,"temp.joinfile") | fields ("junk", "2", >> "temp.joinfile2")
  type("temp.joinfile2") | average >> "temp.joinfile3"
  xshiftnew=real(average.sum)
  del("temp.joinfile?",ver-)
  grep(rrefimage,"temp.joinfile") | fields ("junk", "4", >> "temp.joinfile2")
  type("temp.joinfile2") | average >> "temp.joinfile3"
  yshiftnew=real(average.sum)
  del("temp.joinfile?",ver-)
# Want to take the calculated shift ($2 for x), cancel out the newly calculated
# reference image shift ($2 - xshiftnew = -xnew+$2 = -1*xnew+$2) and add in
# the old reference image shift (xold+(-1*xnew+$2)
	filecalc("temp.joinfile",xxshiftold//"+(-1*"//xshiftnew//"+$2);"//yyshiftold//"+(-1*"//yshiftnew//"+$4)",format="%8.2f%8.2f  ", >> ooffname)
#	filecalc("temp.joinfile",xxshiftold//"+$2;"//yyshiftold//"+$4",format="%8.2f%8.2f  ", >> ooffname)
}
else{
fields("temp.joinfile", "2,4", >> ooffname)
}
	del("temp.joinfile*", ver-)
	list=ooffname
        while(fscan(list,x,y) !=EOF) {
                print(int(x),int(y), >> iintname)
        }
	join(iintname,ooffname,out="temp.joinfile")
	filecalc("temp.joinfile","$3-$1;$4-$2",forma="%6.2f%6.2f", >> ffracname)
        filecalc(ooffname,"$1;$2",format="%4.0f %4.0f", >> rroundname)
        print(" ", >> logoffname)
        print("#Final offsets:", >> logoffname)
        print(" ", >> logoffname)
        type(ooffname, >> logoffname)
#        del(ooffname, ver-)
	del("temp.joinfile", ver-)

# End of "if doshift"

}

# Find & insert header parameter "SCALEFAC" used by imcombine
print("===) Finding scales for coadding images...")
file_of_air=mktemp("foa")
hsel("@"//fflatlist,"$I,AIRMASS,"//eexpname,yes, >> file_of_air)
list=file_of_air
while (fscan (list,img,air,texp) != EOF) {
	imgets(img,"SCALEFAC",>&"dev$null")
	if(imgets.value=="0"){
		sf=10**(0.4*eextcoeff*(air-aairzero))
        	hedit(img,"AIRSCALE",sf,add+,ver-,show-,update+)
        	sf=texp*sf
		imgets(img,"scalefac")
        	hedit(img,"SCALEFAC",sf,add+,ver-,show-,update+)
	} else {
		print("Image "//img//" already has SCALEFAC header keyword.")
	}
}
del(file_of_air,ver-)
list=""

# Find & insert header parameter "WEIGHT" used by imcombine
print("===) Finding weights for coadding images...")
list=fflatlist
while(fscan(list,img) !=EOF) {
	imgets(img,"weight",>&"dev$null")
	if(imgets.value=="0"){
		iterstat(img,nsigrej=5.,maxiter=10,print-,ver-)
		imgets(img,t_expname)
		weight=iterstat.median*real(imgets.value)
		hedit(img,"weight",weight,add+,ver-,show-,update+)
	}
}
list=""

if(ffracshift){
	print("===) Creating fractional-pixel-shifted images.")
	sshiftlist=fflatlist//ooutstring
	list=fflatlist
	while(fscan(list,img) !=EOF) {
		fileroot(img,validim+)
		print(fileroot.root//ooutstring//".imh",>>sshiftlist)
	}
	imshift("@"//fflatlist, "@"//sshiftlist, shifts=ffracname, interp="poly5", bound="nearest")
}

if(!access(eexpmap//".imh")){
print("===) Shifting and coadding the images.")
if(ffracshift){
	coaddlist=sshiftlist
	offsetlist=iintname
}
else {
	coaddlist=fflatlist
	offsetlist=rroundname
}
if (makeavg) {
	imcombine("@"//coaddlist, ccoaddname, sigma="", plfile="", combine="average", rej="none", project-, offsets=offsetlist, masktype="none", blank=0., scale="!SCALEFAC", zero="mode", weight="!WEIGHT", statsec="", expname=t_expname, lthresh=-100000, hthresh=100000, lsig=5., hsig=5., nkeep=1, mclip+, rdnoise=t_rdnoise, gain=t_gain, sigscale=0.1, grow=0., logfile='STDOUT', >> logoffname)
}
else {
	if(stdpars){
	imcombine("@"//coaddlist, ccoaddname, sigma="", plfile="", combine="average", rej="ccdclip", project-, offsets=offsetlist, masktype="none", blank=0., scale="!SCALEFAC", zero="mode", weight="!WEIGHT", statsec="", expname=t_expname, lthresh=-100000, hthresh=100000, lsig=5., hsig=5., nkeep=1, mclip+, rdnoise=t_rdnoise, gain=t_gain, sigscale=0.1, grow=0., logfile='STDOUT', >> logoffname)
	}
	else{
		imcombine("@"//coaddlist, ccoaddname, sigma=t_sigma, plfile=t_plfile, combine=t_combine, reject=t_reject, project-, offsets=offsetlist, masktype=t_masktyp, maskvalue=t_maskval, blank=t_blank, scale=t_scale, zero=t_zero, weight=t_weight, statsec=t_statsec, expname=t_expname, lthresh=t_lthresh, hthresh=t_hthresh, nlow=t_nlow, nhigh=t_nhigh, nkeep=t_nkeep, mclip+, lsigma=t_lsigma, hsigma=t_hsigma, rdnoise=t_rdnoise, gain=t_gain, sigscale=t_sigscale, pclip=t_pclip, grow=t_grow, logfile='STDOUT', >> logoffname)
	}
}
hedit(ccoaddname,"title","Coadded image from "//coaddlist//" using "//offsetlist,add+,ver-,show-,update+)
	iterstat(ccoaddname,nsigrej=5.,maxiter=10.,print-,ver-)
	fileroot(ccoaddname,validim+)
	imar(ccoaddname,"-",iterstat.median,fileroot.root//"sub",title="Sky-subtracted version of "//ccoaddname,ver-)
# Set airmass of coadded image to "effective airmass"
if(access(coaddlist//".airmass.tmp"))del(coaddlist//".airmass.tmp",ver-)
if(access(coaddlist//".airmass.tmp2"))del(coaddlist//".airmass.tmp2",ver-)
hsel("@"//coaddlist, "airmass", "yes", >> coaddlist//".airmass.tmp")
type(coaddlist//".airmass.tmp" | average opt="new_sample", >> coaddlist//".airmass.tmp2")
effairmass=real(average.mean)
hedit(ccoaddname,"airmass",(effairmass),add+,ver-,show-,update+)
del (coaddlist//".airmass.tmp*", ver-)
}
else{
  print("Output coadded image already exists-- new one not created.")
}

# Create exposure maps if the output coadded-image exposure map does not exist.
# The routine should allow for images of different exposure times and sizes,
# while at the same time cutting down on the number of images that need to be
# created if many input images have the same size and exposure time.
if(!access(eexpmap//".imh")){
        print("===) Creating exposure maps...")
	subimgold="_oops"
	testold=0.
	test1old=0
	test2old=0
	list=coaddlist
	explist=coaddlist//"exp"
	while (fscan (list, img) !=EOF) {
		fileroot(img,validim+)
                imgets(img,"i_naxis1")
		test1=int(imgets.value)
		imgets(img,"i_naxis2")
 		test2=int(imgets.value)
		imgets(img,eexpname)
		exposure=real(imgets.value)
		if(test1!=test1old || test2!=test2old || exposure!=testold) {
			subimg = mktemp(fileroot.root)
			print (subimg, >> explist)
			imcalc(img,subimg,"im1*0+"//exposure,ver-)
			test1old=test1
			test2old=test2
			testold=exposure
			subimgold=subimg
		}
		else {
			print (subimgold, >> explist)
		}
	}

# Create coadded-image exposure map
	print("===> Creating coadded-image exposure map...")
	imcombine("@"//explist, "junk"//eexpmap, sigma="", plfile=eexpmap//"pl", combine="average", reject="none", project-, offsets=rroundname, masktype="none", scale="none", zero="none", weight="none", expname=eexpname, mclip+, logfile='STDOUT', >> logoffname)
	imgets("junk"//eexpmap,"ncombine")
	nncombine=int(imgets.value)
	imdel("junk"//eexpmap, ver-, >& "dev$null")
	imdel("@"//explist, ver-, >& "dev$null")
	del(explist, ver-)

# Following should yield accurate exposure map.  We use the plfile from the
# above imcombining, not the actual imcombined image.
# Step 1: just to be safe, get rid of any possible negative values
	imreplace(eexpmap//"pl",0.,lower=INDEF,upper=0.)
# Step 2: Since the plfile lists # images *not* used, we subtract total # images
	imar(eexpmap//"pl","-",nncombine,eexpmap//".imh")
# Step 3: Then multiplying by -1 gives # of images used (positive quantity)
	imar(eexpmap,"*",-1.,eexpmap)
	imdel(eexpmap//"pl.pl",ver-)
	hedit(eexpmap,"title","Exposure map image for coadded image "//ccoaddname,ver-,show-,update+)
}
else{
	print("Output exposure map already exists-- new one not created.")
}

# Create *.imshifts file, listing images & offsets used by masklet task
print("===> Creating output imshifts file...")
	join(coaddlist,rroundname,output=ccoaddname//".imshiftt_tmp")
# Find minimum x and y shift values and subtract them from output offsets
        listimshift=ccoaddname//".imshiftt_tmp"
	list=listimshift
	xmin=100000
	ymin=100000
        while (fscan (list,img,newx,newy) != EOF) {
                if (newx < xmin)  {
                        xmin = newx
                }
                if (newy < ymin)  {
                        ymin = newy
                }
	}
	newx=0
	newy=0
	list=listimshift
        while (fscan (list,img,oldx,oldy) != EOF) {
		newx = oldx - xmin
		newy = oldy - ymin
		print(img, " ", newx, newy, >> ccoaddname//".imshifts")	
	}
	del(ccoaddname//".imshiftt_tmp",ver-)

# Set edges of sky-subtracted image to zero.
fileroot(ccoaddname,validim+)
imar(fileroot.root//"sub","*",eexpmap,fileroot.root//"sub")
imar(fileroot.root//"sub","/",eexpmap,fileroot.root//"sub",divzero=0.)

print("===> All done.  *sub.imh can be used with mkobjmask.")
}
else{
   print("WARNING: task(s) filecalc, fileroot, imcalc and/or iterstat are not")
   print("loaded!  Please load the unloaded task(s) and restart this task.")
}

beep

end

