# sigmanorm.cl -- normalize image by the square root of its exposure map
#
# Renormalizes mosaic image produced by DIMSUM by the square root of
# the exposure map to produce an image which should have uniform rms
# pixel-to-pixel noise across the entire exposed area.
#
# 930726: created by Mark Dickinson
# 940106: Modified to allow user to specify name of normalization image.
# 960328: added addition of header comments to output image
# 960405: changed default normimage suffix from "exp" to "em"

procedure sigmanorm(image)

string	image	  {prompt="Name of image to renormalize"}
string	normimage {"",prompt="Normalization image (default is root//em)"}
string	prefix	  {"norm", prompt="Prefix for renormalized image"}
# bool	undo	  {no,prompt="Undo (invert) normalization step?"}

begin

	string	img		# equals image
	string	ext		# equals extension
	string	nimg		# normalization image
	real	maxexp		# peak exposure time

# Get query parameter.

	img = image

# Strip off filename extension.

	fileroot (img,validim+)
	img = fileroot.root
	ext = fileroot.extension
	if(ext!="")ext="."//ext

# Check to see if normimage is null;  if so, exposure map is assumed to be 
# imageroot//"em"//ext

	if (normimage == "") {
		nimg = img//"em"//ext
	   } else {
		nimg = normimage
	}

# Calculate sqrt of exposure map and multiply into image with normalization
# such that the maximum exposure time in the normalization image is 1.

	minmax(nimg,force+,update-,ver-)
	maxexp = minmax.maxval
	imarith(nimg,"/",maxexp,prefix//img//ext,ver-)
	imfunc(prefix//img,prefix//img,"sqrt",ver-)
# 	if(undo){imarith(img,"/",prefix//img,prefix//img,pix="real",calc="real",ver-,noact-)
# 	}else{
	imarith(img,"*",prefix//img,prefix//img,pix="real",calc="real",ver-)
	if(deftask("addcomment")){
		addcomment(prefix//img,"phiirs.sigmanorm:",ver-)
		addcomment(prefix//img,img//" normalized by "//normimage,ver-)
	}


end

