# trim -- create constant-rms image with a certain minimum exposure time
#
# This task takes an input image and exposure map and creates:
#
# "trim"//img	   	trimmed image
# "trim"//mask     	exposure map for trimmed image
# "trim"//mask//"mask"  exposed/unexposed area mask for trimmed image
# stats			files of trimmed image RMS values (from maskstat)
#
# 960710: created from batch routine dofocasprep.cl

procedure trim(image,mask,stats)

string image 	{prompt="Image (or image list) to trim"}
string mask  	{prompt="Exposure map or list (default=image//em)"}
string stats 	{prompt="File to contain trimmed image RMS values"}

string out="trim" {prompt="Prefix for trimmed images"}
real mingoodexp=0.36 {prompt="Minimum acceptable exptime (fraction of maximum)"}
struct *list

begin

string img, exp, newexp, pprefix, trimlist, imglist, trimmask, explist
string sstats, joinlist, checki, checke, ext
real mmingoodexp
bool doren

img = image
	checki=substr(img,1,1)
	if(checki=="@"){
		imglist=substr(img,2,strlen(img))
		trimlist=imglist//"trim"
	} else {
		imglist=mktemp("imglist")
		trimlist=imglist//"trim"
		print (img, >> imglist)
	}
exp = mask
	checke=substr(exp,1,1)
	if(checke=="@"){
		explist=substr(exp,2,strlen(exp))
	} else {
		explist=mktemp("explist")
		if(checke=="") {
			fileroot (img, validim+)
			print (fileroot.root//"em", >> explist)
		} else {
                	print (exp, >> explist)
		} 
	} 
sstats=stats

pprefix=out
	if(pprefix=="")pprefix="trim"
mmingoodexp=mingoodexp

# join image and mask lists
joinlist=mktemp("joinlist")
join (imglist, explist, >> joinlist)
list=joinlist

while (fscan (list,img,exp) != EOF) {
	fileroot(img, validim+)
	img = fileroot.root
	fileroot(exp, validim+)
	exp = fileroot.root
	ext = fileroot.extension
	if(ext!="") ext="."//ext
	newexp = pprefix//fileroot.root
# the core
    print ("Preparing exposure map...")
	minmax (exp, force+, update+, ver-)
	imar (exp, "/", minmax.maxval, newexp, ver-, noact-)
	imreplace (newexp, 0., lower=INDEF, upper=mmingoodexp)
	imreplace (newexp, 1., lower=mmingoodexp-0.01, upper=INDEF)
	addring (newexp, newexp, ngrow=5, badval=1)
	imar (exp, "*", newexp, newexp, ver-, noact-)
    print ("Normalizing image...")
	sigmanorm (img, normim=newexp, prefix=pprefix)
    print ("Getting statistics...")
	print (pprefix//img, >> trimlist)
	imcopy (newexp, pprefix//img//"mask"//ext, ver-)
	imreplace (pprefix//img//"mask"//ext, 1., lower=0.01, upper=INDEF)
	maskstat (pprefix//img, pprefix//img//"mask"//ext, goodv=1, iterstat+, ver-, >> sstats)
}

del (joinlist, ver-)
if (checke!="@") del (explist, ver-)
if (checki!="@") {
	del (imglist, ver-)
	del (trimlist, ver-)
} else {
	print("Output trimmed images are listed in "//trimlist)
}
print("Output trimmed image statistics are listed in "//sstats)

#imar cf1.coadd3em / 3879.999267578125 cf1.coadd3em2
#imreplace cf1.coadd3em2 0. lower=INDEF upper=0.4
#imreplace cf1.coadd3em2 1. lower=0.39 upper=INDEF
#addring cf1.coadd3em2 cf1.coadd3em2 ngrow=5 badval=1
#imar cf1.coadd3em * cf1.coadd3em2 cf1.coadd3em2
#imar cf1.coadd3em2 / 3879.999267578125 cf1.coadd3em2
#sigmanorm cf1.coadd3 normim=cf1.coadd3em2
#imcopy cf1.coadd3em2.imh cf1.coadd3em3.imh
#imreplace cf1.coadd3em3.imh 1. lower=0.10 upper=INDEF
#imren normcf1.coadd3.imh cf1
#maskstat cf1.imh cf1.coadd3em3.imh goodv=1 iterstat+ ver- >> stats

beep

end

